<?php

namespace EventPortal\Domain\Agenda;

use Carbon\Carbon;

class SessionInput
{
    /** @var array */
    private $input;

    /** @var \Carbon\Carbon */
    private $date;

    public function __construct(array $input)
    {
        $this->input = $input;
        $this->date = now();
    }

    public static function from(array $input): self
    {
        return new self($input);
    }

    public function forceDate(Carbon $date): self
    {
        $this->date = $date;

        return $this;
    }

    public function formatForDatabase(): array
    {
        return collect($this->input)
            ->mapWithKeys(function ($value, $key) {
                return [SessionKeyMap::getDatabaseKeyFromBrowser($key) => $value];
            })
            ->map(function ($value, $key) {
                if ($this->notATimeField($key)) {
                    return $value;
                }

                return $this->timeAsHourMinute($value);
            })
            ->toArray();
    }

    public function formatForBrowser(): array
    {
        return collect($this->input)
            ->mapWithKeys(function ($value, $key) {
                return [SessionKeyMap::getBrowserKeyFromDatabase($key) => $value];
            })
            ->map(function ($value, $key) {
                if (self::notATimeField($key)) {
                    return $value;
                }

                return ['startTime' => Carbon::parse($value)->format('h:i A')];
            })
            ->toArray();
    }

    public function getInput(): array
    {
        return $this->input;
    }

    private function notATimeField(string $key): bool
    {
        $timeFields = [
            'starting_at',
            'ending_at',
            'session-start-time',
            'session-end-time',
        ];

        return in_array($key, $timeFields, true) === false;
    }

    /**
     * @param array|string $value
     */
    private function timeAsHourMinute($value): string
    {
        $time = is_array($value) ? $value['startTime'] : $value;
        $time = Carbon::parse($time);

        return Carbon::create(
            $this->date->year,
            $this->date->month,
            $this->date->day,
            $time->hour,
            $time->minute
        );
    }
}
